'This VB Script is a modified version of the "Attributes to Excel" script.
'You can be use it in conjunction with the PowerNET netlister in "Include Attributes" or "Attributes Only" mode to back/forward annotate.
'You can import Part Types and PCB Decals and part location into OrCAD Capture.
'If there are any problems or questions post to listserver.pads.com\talk
'
'Usage:
'USE AT YOUR OWN RISK
'run this script form PowerPCB
'select desired attributes to import/update in OrCAD, apply or OK
'run Tools|Update Properties... in OrCAD Capture
'verify that part properties are as desired
'
'Revision History:
'11MAR2002 - first release
'26JUL2004 - added support for part location

'This VB Script is suitable for PowerPCB. 
'The sample demonstrates the following PowerLogic/PowerPCB VB capabilities:
' - Part Attribute Collection
' - User Dialog with dynamic list boxes
' - Text File Reports
' - Simple Sort function
' 
' For more details, please refer to the PowerLogic/PowerPCB Basic Editor Help File.

Dim AttrsAll() As String
Dim AttrsSelected() As String
Dim parts As objects

Sub Main
	Begin Dialog UserDialog 580,287,"Attributes to OrCAD Capture Update File",.dialogfunc ' %GRID:10,7,1,1
		ListBox 10,28,210,259,AttrsAll(),.List1
		ListBox 350,28,220,259,AttrsSelected(),.List2
		Text 10,7,120,14,"Part Attributes:",.Text1
		Text 430,7,130,14,"Selected Attributes:",.Text2
		PushButton 230,28,110,21,"Add >>",.btnAdd
		PushButton 230,56,110,21,"Add All >>",.btnAddAll
		PushButton 230,98,110,21,"<< Remove",.btnRemove
		PushButton 230,126,110,21,"<< Remove All",.btnRemoveAll
		OKButton 230,231,110,21
		PushButton 230,203,110,21,"Apply",.btnOrCADUpd
		CancelButton 230,259,110,21
		CheckBox 200,7,170,14,"Map Decal to Footprint",.CheckMapFP
	End Dialog
	Dim dlg As UserDialog

	rc = Dialog(dlg)
End Sub

Rem See DialogFunc help topic for more information.
Private Function dialogfunc(DlgItem$, Action%, SuppValue&) As Boolean
	Select Case Action%
	Case 1 ' Dialog box initialization
		FillListOfAttributes
	Case 2 ' Value changing or button pressed
		dialogfunc = True ' Prevent button press from closing the dialog box
		If DlgItem$ = "btnAdd" Then 
			AddOne
		ElseIf DlgItem$ = "btnAddAll" Then 
			AddAll
		ElseIf DlgItem$ = "btnRemove" Then 
			RemoveOne
		ElseIf DlgItem$ = "btnRemoveAll" Then 
			RemoveAll
		ElseIf DlgItem$ = "btnOrCADUpd" Then 
			FillClipboard
		ElseIf DlgItem$ = "OK" Then
			FillClipboard
			dialogfunc = False
		ElseIf DlgItem$ = "Cancel" Then
			dialogfunc = False
		Else
			UpdateButtons
		End If
		
	Case 3 ' TextBox or ComboBox text changed
	Case 4 ' Focus changed
	Case 5 ' Idle
		Rem dialogfunc = True ' Continue getting idle actions
	Case 6 ' Function key
	End Select
End Function

Sub FillListOfAttributes
	Set parts = ActiveDocument.Components
	GetAttributeListWithPartType
	ReDim AttrsSelected(0)
	DlgListBoxArray "List1",AttrsAll()
	UpdateButtons
End Sub

Sub UpdateButtons
	AllCount = UBound(AttrsAll)
	SelCount = UBound(AttrsSelected)
	CurItem1 = DlgValue("List1") 
	CurItem2 = DlgValue("List2") 
	DlgEnable "btnAdd", allCount <> 0 And CurItem1 >= 0
	DlgEnable "btnAddAll", allCount <> 0
	DlgEnable "btnRemove", selCount <> 0 And CurItem2 >= 0
	DlgEnable "btnRemoveAll", selCount <> 0
	DlgEnable "btnOrCADUpd", selCount <> 0
	DlgEnable "OK", selCount <> 0
End Sub

Sub AddOne
	CurItem = DlgValue("List1") 
	Count = UBound(AttrsSelected)
	ReDim Preserve AttrsSelected(Count + 1)
	AttrsSelected(Count + 1) = AttrsAll(CurItem + 1)
	DlgListBoxArray "List2",AttrsSelected()
	DlgValue "List1", CurItem + 1
	UpdateButtons
End Sub

Sub AddAll
	prevCount = UBound(AttrsSelected)
	ReDim Preserve AttrsSelected(0 To prevCount + UBound(AttrsAll))
	For i = 1 To UBound(AttrsAll)
		AttrsSelected(prevCount + i) = AttrsAll(i)
	Next i
	DlgListBoxArray "List2",AttrsSelected()
	UpdateButtons
End Sub

Sub RemoveOne
	CurItem = DlgValue("List2")
	Count = UBound(AttrsSelected)
	For i = CurItem + 2 To Count
		AttrsSelected(i - 1) = AttrsSelected(i)
	Next i
	ReDim Preserve AttrsSelected(Count - 1)
	DlgListBoxArray "List2",AttrsSelected()
	DlgValue "List2", CurItem
	UpdateButtons
End Sub

Sub RemoveAll
	ReDim Preserve AttrsSelected(0)
	DlgListBoxArray "List2",AttrsSelected()
	UpdateButtons
End Sub

Sub FillClipboard
	StatusBarText = "Export Data to OrCAD Capture Update File..."
    ' Create OrCAD Capture Update file
    
	fname = ActiveDocument.FullName
	nm = Left(fname, Len( fname) - 4)
	UpdateFile = nm & ".upd"

    Open UpdateFile For Output As #1

    ' Output data to OrCAD Capture Update file
    Print #1, """{Reference}"""; vbTab;	'Use Reference for gate combatibility
    For i = 1 To UBound(AttrsSelected)
	If DlgValue("CheckMapFP") And AttrsSelected(i) = "PCB Decal" Then
	    Print #1, """" & "PCB Footprint" & """"; vbTab;
	ElseIf AttrsSelected(i) = "PositionX" Then
		Print #1, """" & "PCB_X" & """"; vbTab;
	ElseIf AttrsSelected(i) = "PositionY" Then
		Print #1, """" & "PCB_Y" & """"; vbTab;
	ElseIf AttrsSelected(i) = "Orientation" Then
		Print #1, """" & "PCB_ORI" & """"; vbTab;
	ElseIf AttrsSelected(i) = "Side" Then
		Print #1, """" & "PCB_SIDE" & """"; vbTab;
	Else
	    	Print #1, """" & AttrsSelected(i) & """"; vbTab;
	End if
    Next i
    Print #1
    For Each aPart In parts
    	Print #1, """" & aPart & """"; vbTab; 
    	Set attrs = aPart.Attributes
    	For i = 1 To UBound(AttrsSelected)
    		If AttrsSelected(i) = "Part Type" Then
    			Print #1, """" & aPart.PartType & """";
    		ElseIf AttrsSelected(i) = "PCB Decal" Then
    			Print #1, """" & aPart.Decal & """";
    			
    		ElseIf AttrsSelected(i) = "PositionX" Then
    			Print #1, """" & aPart.PositionX & """";
    		ElseIf AttrsSelected(i) = "PositionY" Then
    			Print #1, """" & aPart.PositionY & """";
    		ElseIf AttrsSelected(i) = "Orientation" Then
    			Print #1, """" & aPart.Orientation & """";
    		'ElseIf AttrsSelected(i) = "Layer" Then
    		'	Print #1, """" & aPart.layer & """";
    			
    		'ElseIf AttrsSelected(i) = "COMPLOC" Then
    		'	Print #1, """" & aPart.PositionX & "," & aPart.PositionY & """";
    		'ElseIf AttrsSelected(i) = "COMPROT" Then
    		'	Print #1, """" & aPart.Orientation & """";
    		ElseIf AttrsSelected(i) = "Side" Then
				If aPart.LayerName = "Bottom" Then
		    		Print #1, """" & "Bot" & """";
				Else
		    		Print #1, """" & aPart.LayerName & """";
				End If
    		Else
	    		Set attr = attrs(AttrsSelected(i))
    			If Not attr Is Nothing Then
    				Print #1, """" & attr.value & """";
    			Else
    				Print #1, """""";
    			End If
    		End If
   			If i <> UBound(AttrsSelected) Then Print #1, vbTab; 
     	Next i
	    Print #1
    Next aPart
    ' Close the OrCAD Capture Update file
    Close #1
    Shell "Write " & Chr(34) & UpdateFile & Chr(34), 1
	StatusBarText = ""
End Sub

Public Sub GetAttributeListWithPartType()
	StatusBarText = "Scanning all parts for attribute Info..."
	' calculate total count of attributes
	totalCount = 0
	For Each obj In parts
		totalCount = totalCount + obj.Attributes.Count
	Next
	' fill string array of attribute names avoiding duplications	
	'MyCount = 2
	MyCount = 6
	ReDim AttrsAll(0 To totalCount + 1)
	AttrsAll(1) = "Part Type"
	AttrsAll(2) = "PCB Decal"
	AttrsAll(3) = "PositionX"
	AttrsAll(4) = "PositionY"
	AttrsAll(5) = "Orientation"
	AttrsAll(6) = "Side"
'	AttrsAll(6) = "Layer"
	
'	AttrsAll(7) = "COMPLOC"
'	AttrsAll(8) = "COMPROT"
'	AttrsAll(9) = "COMPSIDE"
	
	For Each nextObj In parts
		Set attrs = nextObj.Attributes
		For Each nextAttr In attrs
			bFound = False
			'avoid duplications
			For i=1 To myCount
				If AttrsAll(i) = nextAttr.Name Then
					bFound=True
					Exit For
				End If
			Next i
			If bFound=False And nextAttr.Name <> "Source Package" Then	'Source Package cannot be changed as property
				myCount = myCount+1
				AttrsAll(myCount) = nextAttr.Name
			End If
		Next nextAttr
	Next nextObj
	If (MyCount = 0) Then
		ReDim  AttrsAll(0)
	Else
		ReDim Preserve AttrsAll(MyCount)
	End If
	'sort list
	SortStringArray AttrsAll()
	StatusBarText = ""
End Sub

'Sort String Array
Public Sub SortStringArray(ByRef strList$())
	lo = 1
	hi = UBound(strList)
    While hi > lo
        max = lo
        For p = lo+1 To hi
			If StrComp(strList(p), strList(max), 1) > 0 Then
                max = p
            End If
		Next
		'swap max with hi
		temp = strList(max)
		strList(max) = strList(hi)
		strList(hi) = temp
		hi = hi - 1
    Wend
End Sub
